#include <assert.h>
#include <string.h>

#include "objLevel.h"
#include "measures.h"
#include "display.h"
#include "win.h"
#include "gc.h"
#include "error.h"

#define MAXLVLS 20

static void update (Object_Ptr) ;
static void draw (Object_Ptr) ;
static void parse (Object_Ptr, int, FILE *) ;

static Object_Methods LevelMethods =
{
    update,
    draw,
    parse
} ;

typedef struct Object__
{
    Object_MethodsPtr methods ;
    int valIndex ;
    int x ;
    int y ;
    int width ;
    int ticks ;
    double normal ;
    double deltaWarning ;
    double deltaAlarm ;
    double scale ;
}
Object ;

static Object Objects [MAXLVLS] ;
static int NbObjects = 0 ;

extern Object_Ptr Level_Create (void)
{
    Object_Ptr obj ;
    assert (NbObjects < MAXLVLS) ;
    obj = Objects + NbObjects++ ;

    obj->methods = & LevelMethods ;
    obj->valIndex = -1 ;
    obj->x = 0 ;
    obj->y = 0 ;
    obj->width = 0 ;
    obj->ticks = 0 ;
    obj->normal = 0 ;
    obj->deltaWarning = 0 ;
    obj->deltaAlarm = 0 ;
    obj->scale = 0 ;

    return obj ;
}

static void update (Object_Ptr obj)
{
    /* nothing */
}

static void draw (Object_Ptr obj)
{
    double delta ;
    int tick ;
    GC color ;
    int y ;

    delta = Measure_Get (obj->valIndex) - obj->normal ;

    if (delta <= - obj->deltaAlarm || obj->deltaAlarm <= delta)
    {
	color = AnalogAlertGC ;
    }
    else
    if (delta <= - obj->deltaWarning || obj->deltaWarning <= delta)
    {
	color = AnalogWarningGC ;
    }
    else
    {
	color = AnalogGoodGC ;
    }

    tick = (int)(delta * obj->scale) ;

    if (tick < - obj->ticks)
    {
	tick = - obj->ticks ;
    }
    else
    if (tick >   obj->ticks)
    {
	tick =   obj->ticks ;
    }

    /* lower vertical coordinates are upside -- hence minus ticks */
    y = obj->y - 2*tick ;

    XDrawLine (Dpy, Panel, color, obj->x, y, obj->x + obj->width - 1, y) ;
}

static void parse (Object_Ptr obj, int valIndex, FILE * f)
{
    char c ;

    assert (obj != NULL) ;
    assert (obj->methods == & LevelMethods) ;

    obj->valIndex = valIndex ;

    while (fscanf (f, " %c", & c) == 1)
    {
	if (c == ';')
	{
	    /* finished */
	    break ;
	}
	else
	{
	    char name [10] ;
	    ungetc (c, f) ;

	    if (fscanf (f, "%9[a-z] = ", name) != 1)
	    {
		error ("level: expected <name>=...") ;
	    }
	    else
	    if (strcmp (name, "pos") == 0)
	    {
		if (fscanf (f, "%d , %d ", & obj->x, & obj->y) != 2)
		{
		    error ("level: expected pos=<integer>,<integer>") ;
		}
	    }
	    else
	    if (strcmp (name, "width") == 0)
	    {
		if (fscanf (f, "%d ", & obj->width) != 1)
		{
		    error ("level: expected width=<integer>") ;
		}
	    }
	    else
	    if (strcmp (name, "ticks") == 0)
	    {
		if (fscanf (f, "%d ", & obj->ticks) != 1)
		{
		    error ("level: expected ticks=<integer>") ;
		}
	    }
	    else
	    if (strcmp (name, "limits") == 0)
	    {
		if (fscanf (f, "%lf , %lf , %lf ", & obj->normal,
		    & obj->deltaWarning, & obj->deltaAlarm) != 3)
		{
		    error ("level: expected limits=<normal>,<warn>,<alarm>") ;
		}
	    }
	    else
	    {
		error ("level: unknown syntax '%s ='", name) ;
	    }
	}
    }

    if (c != ';')
    {
	error ("level: unexpected end-of-file") ;
    }

    /*
    compute scale such that alarm level is the farthest tick from center
    */
    obj->scale = (double)obj->ticks / obj->deltaAlarm ;
}

