#include <assert.h>
#include <string.h>
#include <math.h>

#include "objSpeedo.h"
#include "measures.h"
#include "display.h"
#include "win.h"
#include "gc.h"
#include "error.h"

#define MAXSPDS 20

static void update (Object_Ptr) ;
static void draw (Object_Ptr) ;
static void parse (Object_Ptr, int, FILE *) ;

static Object_Methods SpeedoMethods =
{
    update,
    draw,
    parse
} ;

typedef struct Object__
{
    Object_MethodsPtr methods ;
    int valIndex ;
    int x ;
    int y ;
    int intRay ;
    int extRay ;
    int normal ;
    int deltaWarning ;
    int deltaAlarm ;
}
Object ;

static Object Objects [MAXSPDS] ;
static int NbObjects = 0 ;

/* angles in  [0..90] -> values in [0..1000] */
static int CosTab [91] = {0,} ;

static void InitCosTab (void)
{
    if (CosTab[0] == 0) /* should be one once initialized */
    {
	int i ;
	double degreesToRadians ;

	degreesToRadians = acos (0.0L) / 90.0L ;

	for (i=0; i<=90; i++)
	{
	    /* round to nearest */
	    CosTab[i] = floor (1000.0L * cos (i * degreesToRadians) + 0.5L) ;
	}
    }
}

extern Object_Ptr Speedo_Create (void)
{
    Object_Ptr obj ;
    assert (NbObjects < MAXSPDS) ;
    obj = Objects + NbObjects++ ;

    obj->methods = & SpeedoMethods ;
    obj->valIndex = -1 ;
    obj->x = 0 ;
    obj->y = 0 ;
    obj->intRay = 0 ;
    obj->extRay = 0 ;
    obj->normal = 0 ;
    obj->deltaWarning = 0 ;
    obj->deltaAlarm = 0 ;

    return obj ;
}

static void update (Object_Ptr obj)
{
    /* nothing */
}

static void draw (Object_Ptr obj)
{
    int delta, absDelta ;
    int angle ;
    int x1, y1, x2, y2 ;
    int cosVal, sinVal ;
    int xOfs ;
    GC color ;

    delta = Measure_Get (obj->valIndex) - obj->normal ;
    absDelta = abs (delta) ;

    if (absDelta >= obj->deltaAlarm)
    {
	color = AnalogAlertGC ;
	absDelta = obj->deltaAlarm ;
    }
    else
    if (absDelta >= obj->deltaWarning)
    {
	color = AnalogWarningGC ;
    }
    else
    {
	color = AnalogGoodGC ;
    }

    angle = (90 * absDelta) / obj->deltaAlarm ;
    assert (0 <= angle && angle <= 90) ;
    InitCosTab () ; /* actually done only once */
    cosVal = CosTab [angle] ;
    sinVal = CosTab [90-angle] ;
    xOfs = +500 ;

    if (delta < 0)
    {
	sinVal = - sinVal ;
	xOfs = - xOfs ;
    }

    x1 = obj->x + (sinVal * obj->intRay + xOfs)/1000 ;
    y1 = obj->y - (cosVal * obj->intRay +500)/1000 ;

    x2 = obj->x + (sinVal * obj->extRay + xOfs)/1000 ;
    y2 = obj->y - (cosVal * obj->extRay +500)/1000 ;

    XDrawLine (Dpy, Panel, color, x1, y1, x2, y2) ;
}

static void parse (Object_Ptr obj, int valIndex, FILE * f)
{
    char c ;

    assert (obj != NULL) ;
    assert (obj->methods == & SpeedoMethods) ;

    obj->valIndex = valIndex ;

    while (fscanf (f, " %c", & c) == 1)
    {
	if (c == ';')
	{
	    /* finished */
	    break ;
	}
	else
	{
	    char name [10] ;
	    ungetc (c, f) ;

	    if (fscanf (f, "%9[a-z] = ", name) != 1)
	    {
		error ("speedo: expected <name>=...") ;
	    }
	    else
	    if (strcmp (name, "pos") == 0)
	    {
		if (fscanf (f, "%d , %d ", & obj->x, & obj->y) != 2)
		{
		    error ("speedo: expected pos=<integer>,<integer>") ;
		}
	    }
	    else
	    if (strcmp (name, "r") == 0)
	    {
		if (fscanf (f, "%d , %d ", & obj->intRay, & obj->extRay) != 2)
		{
		    error ("speedo: expected r=<integer>,<integer>") ;
		}
	    }
	    else
	    if (strcmp (name, "limits") == 0)
	    {
		if (fscanf (f, "%d , %d , %d ", & obj->normal,
		    & obj->deltaWarning, & obj->deltaAlarm) != 3)
		{
		    error ("speedo: expected limits=<normal>,<warn>,<alarm>") ;
		}
	    }
	    else
	    {
		error ("speedo: unknown syntax '%s ='", name) ;
	    }
	}
    }

    if (c != ';')
    {
	error ("speedo: unexpected end-of-file") ;
    }
}

