#include <assert.h>
#include <string.h>

#include "objThermo.h"
#include "measures.h"
#include "display.h"
#include "win.h"
#include "gc.h"
#include "config.h"
#include "error.h"

#define MAXTHMS 20

static void update (Object_Ptr) ;
static void draw (Object_Ptr) ;
static void parse (Object_Ptr, int, FILE *) ;

static Object_Methods ThermoMethods =
{
    update,
    draw,
    parse
} ;

typedef struct Object__
{
    Object_MethodsPtr methods ;
    int valIndex ;
    int x ;
    int y ;
    int width ;
    int height ;
    int bottom ;
    int top ;
    int warning ;
    int alarm ;
    int warningHeight ;
    int alarmHeight ;
    int scaleDrawn ;
}
Object ;

static Object Objects [MAXTHMS] ;
static int NbObjects = 0 ;

extern Object_Ptr Thermo_Create (void)
{
    Object_Ptr obj ;
    assert (NbObjects < MAXTHMS) ;
    obj = Objects + NbObjects++ ;

    obj->methods = & ThermoMethods ;
    obj->valIndex = -1 ;
    obj->x = 0 ;
    obj->y = 0 ;
    obj->width = 0 ;
    obj->height = 0 ;
    obj->bottom = 0 ;
    obj->top = 0 ;
    obj->warning = 0 ;
    obj->alarm = 0 ;
    obj->warningHeight = 0 ;
    obj->alarmHeight = 0 ;
    obj->scaleDrawn = 0 ;

    return obj ;
}

static void update (Object_Ptr obj)
{
    assert (obj != NULL) ;
    assert (obj->methods == & ThermoMethods) ;

    if (obj->scaleDrawn) return ;

    if (obj->height >= 20)
    {
	int ticks ;
	int delta ;
	int i ;
	int x, y ;
	char tmp [10] ;

	ticks = obj->height / 20 ;
	delta = obj->top - obj->bottom ;

	for (i=0; i<=ticks; i++)
	{
	    x = obj->x + 8 ;
	    y = obj->y + 1 - 20*i ;
	    sprintf (tmp, TempFmt, obj->bottom + ((i * delta) / ticks)) ;
	    XDrawString (Dpy, EmptyPanel, TextGC, x, y, tmp, strlen (tmp)) ;
	}
    }

    obj->scaleDrawn = 1 ;
}

static void draw (Object_Ptr obj)
{
    int height, left ;
    GC color ;
    int y ;

    assert (obj != NULL) ;
    assert (obj->methods == & ThermoMethods) ;

    height = (Measure_Get (obj->valIndex) - obj->bottom) * obj->height
	/ (obj->top - obj->bottom) ;
    color = AnalogGoodGC ;
    left = height ;

    if (height >= obj->warningHeight)
    {
	y = obj->y - obj->warningHeight ;
	left = height - obj->warningHeight ;
	XFillRectangle (Dpy, Panel, color, obj->x, y, obj->width,
	    obj->warningHeight) ;
	color = AnalogWarningGC ;
    }

    if (height >= obj->alarmHeight)
    {
	y = obj->y - obj->alarmHeight ;
	left = height - obj->alarmHeight ;
	XFillRectangle (Dpy, Panel, color, obj->x, y, obj->width,
	    obj->alarmHeight - obj->warningHeight) ;
	color = AnalogAlertGC ;
    }

    if (left >= 0)
    {
	y = obj->y - height ;
	XFillRectangle (Dpy, Panel, color, obj->x, y, obj->width, left) ;
    }
}

static void parse (Object_Ptr obj, int valIndex, FILE * f)
{
    char c ;

    assert (obj != NULL) ;
    assert (obj->methods == & ThermoMethods) ;

    obj->valIndex = valIndex ;

    while (fscanf (f, " %c", & c) == 1)
    {
	if (c == ';')
	{
	    /* finished */
	    break ;
	}
	else
	{
	    char name [10] ;
	    ungetc (c, f) ;

	    if (fscanf (f, "%9[a-z] = ", name) != 1)
	    {
		error ("thermo: expected <name>=...") ;
	    }
	    else
	    if (strcmp (name, "pos") == 0)
	    {
		if (fscanf (f, "%d , %d ", & obj->x, & obj->y) != 2)
		{
		    error ("thermo: expected pos=<integer>,<integer>") ;
		}
	    }
	    else
	    if (strcmp (name, "size") == 0)
	    {
		if (fscanf (f, "%d , %d ", & obj->width, & obj->height) != 2)
		{
		    error ("thermo: expected size=<integer>,<integer>") ;
		}
	    }
	    else
	    if (strcmp (name, "limits") == 0)
	    {
		if (fscanf (f, "%d , %d , %d , %d ", & obj->bottom,
		    & obj->warning, & obj->alarm, & obj->top) != 4)
		{
		    error ("thermo: expected limits=<bottom>,<warning>,"
			"<alarm>,<top>") ;
		}
	    }
	    else
	    {
		error ("thermo: unknown syntax '%s ='", name) ;
	    }
	}
    }

    if (c != ';')
    {
	error ("thermo: unexpected end-of-file") ;
    }

    /*
    compute scale such that height ranges from bottom to top
    */
    obj->warningHeight = (obj->warning - obj->bottom) * obj->height
	/ (obj->top - obj->bottom) ;
    obj->alarmHeight = (obj->alarm - obj->bottom) * obj->height
	/ (obj->top - obj->bottom) ;
}

